#!/bin/ksh

# Copyright 2006-2015 Carl Thijssen

# This file is part of the Ultimate Deployment Appliance.
#
# Ultimate Deployment Appliance is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Ultimate Deployment Appliance is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

PATCHDIR=$1

VERSION=2.0
BUILD=25
LOGFILE=$VERSION\_$BUILD.$$.log

PUBLIC=/var/public
LOGDIR=/tmp
BINDIR=$PUBLIC/bin
CGIBINDIR=$PUBLIC/cgi-bin
CGIBINOSDIR=$PUBLIC/cgi-bin/os
CGIBINSVCDIR=$PUBLIC/cgi-bin/services
WWWDIR=$PUBLIC/www
WWWSUPDIR=$PUBLIC/www/supportmatrix
WWWSUPVMDIR=$PUBLIC/www/supportmatrix/vmware
ICONS=$WWWDIR/icon
JSDIR=$WWWDIR/js
TEMPLATES=$WWWDIR/templates
CONFDIR=$PUBLIC/conf
CONFMOUNTSDIR=$PUBLIC/conf/mounts
VERSIONDIR=$CONFDIR/version
TFTPROOT=$PUBLIC/tftproot
FILESDIR=$PUBLIC/files
PATCHTOPDIR=$PUBLIC/patches

if [ "$PATCHDIR" == "" ]
then
  PATCHDIR=$PATCHTOPDIR/patch.$VERSION.$BUILD.$$
else
  cd $PATCHDIR
fi

log()
{
  TIMESTAMP=`date +%Y%m%d-%H%M%S`
  echo $*
  echo $TIMESTAMP $* >> $LOGDIR/$LOGFILE
}


################ copy actions #########################
copy_files_files()
{
  ls -1 files/files | while read i
   do
     copy_file files/files/$i $BINDIR
   done
}

copy_cgi_bin_files()
{
  ls -1 files/cgi-bin | while read i
  do
     copy_file files/cgi-bin/$i $CGIBINDIR
  done
}

copy_cgi_bin_os_files()
{
  ls -1 files/cgi-bin/os | while read i
  do
     copy_file files/cgi-bin/os/$i $CGIBINOSDIR
  done
}

copy_conf_files()
{
  ls -1 files/conf | while read i
  do
     if [ -f files/conf/$i ]
     then
       copy_file files/conf/$i $CONFDIR
     fi
  done
}

copy_template_files()
{
  ls -1 files/www/templates | while read i
  do
     if [ -f files/www/templates/$i ]
     then
       copy_file files/www/templates/$i $TEMPLATES
     fi
  done
}

copy_js_files()
{
  ls -1 files/www/js | while read i
  do
     if [ -f files/www/js/$i ]
     then
       copy_file files/www/js/$i $JSDIR
     fi
  done
}


copy_file()
{
  FILE=$1
  DSTDIR=$2
  BASEFILE=`basename $FILE`
  log Copying file $FILE to $DSTDIR
  echo -n "  $FILE in $DSTDIR"
  diff $FILE $DSTDIR/$FILE > /dev/null 2>&1
  if [ $? -eq 0 ]
  then
    log Keeping Current version
    echo " OK! (current version of $FILE is ok)"
  else 
    log Backing up file to $PATCHDIR/before/$BASEFILE
    cp -f -p $DSTDIR/$FILE $PATCHDIR/before/$BASEFILE
    log Replacing $DSTDIR/$BASEFILE for new version $FILE
    cp -f -p $FILE $DSTDIR/$BASEFILE
    if [ $? -ne 0 ]
    then
      log ERROR: Replace Failed
      if [ ! -d $DSTDIR ]
      then
        log ERROR: $DSTDIR does not exist
        echo "FAILED! (Destination dir $DSTDIR does not exist)"
      else
        if [ ! -f $FILE ]
        then
          log ERROR: Source file $FILE does not exist
          echo "FAILED! (Source file $FILE does not exist)"
        else
          log ERROR: Unknown error
          echo "FAILED!"
        fi
      fi
    else
      log OK: Copied $FILE to $DSTDIR/$BASEFILE
      cp -f -p $DSTDIR/$FILE $PATCHDIR/after/$BASEFILE
      log Copied $FILE to $PATCHDIR/after/$BASEFILE
    fi
  fi
}

overwrite_conf_files()
{
  log Overwrite conf files
  for file in \
      os.conf 
  do
     if [ -f $CONFDIR/$file ]
     then
      BACKUP=$file.$VERSION.$BUILD.backup.$$
      log Found current file $file, backing it up
      cp -f -p $CONFDIR/$file $CONFDIR/$BACKUP
      cp -f -p $CONFDIR/$file $PATCHDIR/before/$BACKUP
     fi
     log Overwriting $file
     cp -f -p $CONFDIR/$file.overwrite $CONFDIR/$file
     if [ $? -eq 0 ]
     then
       log OK replaced $file
       cp -f -p $CONFDIR/$file $PATCHDIR/after/$BACKUP
       if [ $? -eq 0 ] 
       then
         log Removing .overwrite file
         rm $CONFDIR/$file.overwrite
       fi
     else
       log FAILED to replace file $CONFDIR/$file with $CONFDIR/$file.overwrite
     fi
  done
}

change_file_permissions()
{
  log Changing permissions on public directory
  chmod -R 755 /var/public/bin
  chown -hR apache.apache /var/public/bin

  chmod -R 755 /var/public/cgi-bin
  chown -hR apache.apache /var/public/cgi-bin

  chmod -R 755 /var/public/www/templates
  chown -hR apache.apache /var/public/www/templates

  chmod -R 755 /var/public/conf
  chown -hR apache.apache /var/public/conf
}

############################   Main routines ########################
prepare_upgrade()
{
  log Preparing upgrade...

  mkdir -p $PATCHDIR
  if [ $? -eq 0 ]
  then
    log OK: Created patchdir $PATCHDIR
    cd $PATCHDIR
  else
    log ERROR: Could not create patchdir $PATCHDIR
    exit 2
  fi

  for newdir in $PATCHDIR/before $PATCHDIR/after $PATCHDIR/version
  do
    log Creating directory $newdir
    mkdir $newdir
    if [ $? -eq 0 ]
    then
      log OK: Created directory $newdir
    else
      log ERROR: Could not create $newdir
      exit 1
    fi
  done

}

determine_current_versions()
{
  log Determining current version
  ls -1 $VERSIONDIR/*.dat | while read filename
  do
    log Found version file $filename
    cp -f $filename $PATCHDIR/version 
    if [ $? -eq 0 ]
    then
      log Copied version file $filename to $PATCHDIR/version
    else
      log Could not copy version file $filename to $PATCHDIR/version
    fi
  done
}

install_packages()
{
 log Installing Packages
 yum --nogpgcheck -y localinstall rpm/*.rpm
 if [ $? -eq 0 ]
 then
   log Install of packages succesfully
 else
   log Failed to install packages
 fi
}

copy_files()
{
  log Copying Files
  copy_files_files
  copy_cgi_bin_files
  copy_cgi_bin_os_files
  copy_conf_files
  copy_template_files
  copy_js_files
}

post_copy_actions()
{
   log Post Copy Actions
   overwrite_conf_files
   install_packages
   change_file_permissions
}

create_version_info()
{
  log Creating Versioninfo 
  VERSIONFILE=$VERSIONDIR/uda$BUILD.dat
  echo VERSION=$VERSION\_Build$BUILD > $VERSIONFILE
  log Wrote $VERSIONFILE
}

wrapup()
{
  WRAPUP=$PATCHTOPDIR/patch.$VERSION.$BUILD.$$.tgz
  log Wrapping up the results to $WRAPUP
  log Copying logfile to $PATCHDIR
  cp -f $LOGDIR/$LOGFILE $PATCHDIR
  tar -czf $WRAPUP $PATCHDIR
  rm -rf $PATCHDIR
  log For logfile see $LOGDIR/$LOGFILE
}

check_prereqs()
{
  log Checking Prereqs

  if [ ! -f $PATCHDIR/version/uda17.dat ]
  then
    log ERROR: Build 17 not found, exiting
    exit 1
  else
    log OK: Build 17 found
  fi

  if [ ! -f $PATCHDIR/version/uda19.dat ]
  then
    log ERROR: Build 19 not found, exiting
    exit 1
  else
    log OK: Build 19 found
  fi

  if [ ! -f $PATCHDIR/version/uda20.dat ]
  then
    log ERROR: Build 20 not found, exiting
    exit 1
  else
    log OK: Build 20 found
  fi

  if [ ! -f $PATCHDIR/version/uda23.dat ]
  then
    log ERROR: Build 23 not found, exiting
    exit 1
  else
    log OK: Build 23 found
  fi

}

############################   Main loop ########################
prepare_upgrade
determine_current_versions
check_prereqs
pre_copy_actions
copy_files
post_copy_actions
create_version_info
wrapup
